//
//  SemiModalView.m
//  SemiModalSample
//
//  Created by 中澤 剛 on 12/11/28.
//  Copyright (c) 2012年 Go Nakazawa. All rights reserved.
//

#import "SemiModalView.h"

#define invisibleHorizontalOnTopCenter CGPointMake(self.frame.origin.x + self.frame.size.width/2, self.frame.origin.y - self.frame.size.height/2)
#define invisibleHorizontalOnBottomCenter CGPointMake(self.frame.origin.x + self.frame.size.width/2, self.frame.origin.y + self.frame.size.height*3/2)
#define invisibleVerticalOnLeftCenter CGPointMake(self.frame.origin.x - self.frame.size.width/2, self.frame.origin.y + self.frame.size.height/2)
#define invisibleVerticalOnRightCenter CGPointMake(self.frame.origin.x + self.frame.size.width*3/2, self.frame.origin.y + self.frame.size.height/2)

@implementation SemiModalView

@synthesize scrollView = _scrollView;
@synthesize usage = _usage;
@synthesize isHiding = _isHiding;
@synthesize selector = _selector;
@synthesize visibleCenter = _visibleCenter;
@synthesize hiddenCenter = _hiddenCenter;

- (id)initWithFrame:(CGRect)frame usage:(int)usage items:(NSArray *)items selector:(SEL)selector
{
    self = [super initWithFrame:frame];
    if (self) {
        self.usage = usage;
        self.selector = selector;
        self.visibleCenter = CGPointMake(self.frame.origin.x + self.frame.size.width/2, self.frame.origin.y + self.frame.size.height/2);
        //set center CGPoint of when hidden
        if (self.usage == SemiModalViewUsageHorizontalOnBottom) {
            self.hiddenCenter = invisibleHorizontalOnBottomCenter;
        }else if (self.usage == SemiModalViewUsageHorizontalOnTop){
            self.hiddenCenter = invisibleHorizontalOnTopCenter;
        }else if (self.usage == SemiModalViewUsageVerticalOnLeft){
            self.hiddenCenter = invisibleVerticalOnLeftCenter;
        }else if (self.usage == SemiModalViewUsageVerticalOnRight){
            self.hiddenCenter = invisibleVerticalOnRightCenter;
        }
        
        self.opaque = NO;
        self.backgroundColor = [UIColor colorWithRed:0.0f green:0.0f blue:0.0f alpha:0.3f];
        
        //set visibility at initialization
        self.center = self.hiddenCenter;
        self.isHiding = YES;
        
        self.scrollView = [[UIScrollView alloc]init];
        self.scrollView.frame = CGRectMake(0, 0, self.frame.size.width, self.frame.size.height);
        [self setItemsOnScrollView:items];
        [self addSubview:self.scrollView];
    }
    return self;
}

- (void)changeVisibility
{
    [UIView animateWithDuration:0.30f animations:^{
        if (self.isHiding) {
            //Make view visible
            self.center = self.visibleCenter;
            self.isHiding = NO;
        }else{
            //make view invisible
            self.center = self.hiddenCenter;
            self.isHiding = YES;
        }
    }];
}

- (void)setItemsOnScrollView:(NSArray *)items
{
    int tagNumber = 0;
    CGFloat contentWidth = 0.0f;
    CGFloat contentHeight = 0.0f;
    for (id item in items) {
        if ([item isKindOfClass:[UIImage class]]) {
            UIImage *itemImage = (UIImage *)item;
            UIButton *itemButton = [UIButton buttonWithType:UIButtonTypeCustom];
            if (itemImage.size.width > self.frame.size.width || itemImage.size.height > self.frame.size.height) {
                CGFloat aspectWidth = self.frame.size.width / itemImage.size.width;
                CGFloat aspectHeight = self.frame.size.height / itemImage.size.height;
                
                CGFloat aspectFactor = 0.0f;
                if (aspectWidth < aspectHeight) {
                    aspectFactor = aspectWidth;
                }else{
                    aspectFactor = aspectHeight;
                }
                itemButton.frame = CGRectMake(contentWidth, contentHeight, itemImage.size.width * aspectFactor, itemImage.size.height * aspectFactor);
            }else{
                itemButton.frame = CGRectMake(contentWidth, contentHeight, itemImage.size.width, itemImage.size.height);
            }
            
            [itemButton setImage:itemImage forState:UIControlStateNormal];
            [itemButton addTarget:self.superview action:self.selector forControlEvents:UIControlEventTouchDown];
            itemButton.tag = tagNumber;
            [self.scrollView addSubview:itemButton];
            
            if (self.usage == SemiModalViewUsageHorizontalOnTop || self.usage == SemiModalViewUsageHorizontalOnBottom) {
                contentWidth = contentWidth + itemButton.frame.size.width;
            }else if (self.usage == SemiModalViewUsageVerticalOnLeft || self.usage == SemiModalViewUsageVerticalOnRight){
                contentHeight = contentHeight + itemButton.frame.size.height;
            }
            tagNumber++;
        }
    }
    if (self.usage == SemiModalViewUsageHorizontalOnTop || self.usage == SemiModalViewUsageHorizontalOnBottom) {
        self.scrollView.contentSize = CGSizeMake(contentWidth, self.frame.size.height);
    }else if (self.usage == SemiModalViewUsageVerticalOnLeft || self.usage == SemiModalViewUsageVerticalOnRight){
        self.scrollView.contentSize = CGSizeMake(self.frame.size.width, contentHeight);
    }
}

/*
 // Only override drawRect: if you perform custom drawing.
 // An empty implementation adversely affects performance during animation.
 - (void)drawRect:(CGRect)rect
 {
 // Drawing code
 }
 */

@end
